/* -------------------------------------------------------------- */
/* (C) Copyright 2001,2006                                       */
/* International Business Machines Corporation,                   */
/* Sony Computer Entertainment Incorporated,                      */
/* Toshiba Corporation.                                           */
/*                                                                */
/* All Rights Reserved.                                           */
/* -------------------------------------------------------------- */
#ifndef _FENV_H
#define _FENV_H	1

enum {
  /* Single precision divide by zero, element 0 */
  FE_DIVBYZERO = 1 << 19,
#define FE_DIVBYZERO	FE_DIVBYZERO
  /* Double precision overflow, element 0 */
  FE_OVERFLOW_DBL = 1 << 13,
#define FE_OVERFLOW_DBL	FE_OVERFLOW_DBL
  /* Double precision underflow, element 0 */
  FE_UNDERFLOW_DBL = 1 << 12,
#define FE_UNDERFLOW_DBL FE_UNDERFLOW_DBL
  /* Double precision inexact result, element 0 */
  FE_INEXACT = 1 << 11,
#define FE_INEXACT	FE_INEXACT

  /* Double precision invalid operation, element 0 */
  FE_INVALID = 1 << 10,
#define FE_INVALID	FE_INVALID
  /* Double precision possibly non-compliant result due to QNaN propogation */
  FE_NC_NAN = 1 << 9,
#define FE_NC_NAN	FE_NC_NAN
  /* Double precision possibly non-compliant result due to denormal operand */
  FE_NC_DENORM = 1 << 8,
#define FE_NC_DENORM	FE_NC_DENORM

  /* Single precision overflow, element 0 */
  FE_OVERFLOW_SNGL = 1 << 2,
#define FE_OVERFLOW_SNGL FE_OVERFLOW_SNGL
  /* Single precision underflow, element 0 */
  FE_UNDERFLOW_SNGL = 1 << 1,
#define FE_UNDERFLOW_SNGL FE_UNDERFLOW_SNGL
  /* Single precision non-zero result could be different than IEEE */
  FE_DIFF_SNGL = 1 << 0,
#define FE_DIFF_SNGL	FE_DIFF_SNGL
};

#define FE_OVERFLOW	(FE_OVERFLOW_SNGL | FE_OVERFLOW_DBL)
#define FE_UNDERFLOW	(FE_UNDERFLOW_SNGL | FE_UNDERFLOW_DBL)
#define FE_ALL_EXCEPT	(FE_DIVBYZERO | FE_OVERFLOW | FE_UNDERFLOW | FE_INEXACT | FE_INVALID | FE_NC_NAN | FE_NC_DENORM | FE_DIFF_SNGL)
#define FE_ALL_EXCEPT_DBL (FE_OVERFLOW_DBL | FE_UNDERFLOW_DBL | FE_INEXACT | FE_INVALID | FE_NC_NAN | FE_NC_DENORM)

/* The SPU supports all four define rounding mode for double precision
 * rounding. Single precision is always rounded towards zero.
 * NOTE: The defines for the rounding mode are for element 0 (scalar)
 *       only.
 */
enum  {
  FE_TONEAREST = 0,
#define FE_TONEAREST	FE_TONEAREST
  FE_TOWARDZERO = 1,
#define FE_TOWARDZERO	FE_TOWARDZERO
  FE_UPWARD = 2,
#define FE_UPWARD	FE_UPWARD
  FE_DOWNWARD = 3
#define FE_DOWNWARD	FE_DOWNWARD
};


/* Type representing exception flags. Contains the entire quadword of the
 * floating-point status control register.
 */
typedef unsigned int fexcept_t;

/* Type representing floating-point environment. Contains the entire quadword
 * of the floating-point status control register.
 */
typedef unsigned int fenv_t;

/* If the default argument is used we use this value.
 */
extern const fenv_t __fe_dfl_env;
#define FE_DFL_ENV	(&__fe_dfl_env)

#endif /* fenv.h */
